Package: bayNorm
Type: Package
Title: Single-cell RNA sequencing data normalization
Version: 1.28.0
Authors@R: c(person("Wenhao", "Tang", role=c("aut", "cre"),
        email="wt215@ic.ac.uk"), 
        person("Fran<U+00E7>ois", "Bertaux", role=c("aut")),
        person("Philipp", "Thomas", role=c("aut")),
        person("Claire", "Stefanelli", role=c("aut")),
        person("Malika", "Saint", role=c("aut")),
        person("Samuel", "Marguerat", role=c("aut")),
        person("Vahid", "Shahrezaei", role=c("aut")))
Description: bayNorm is used for normalizing single-cell RNA-seq data.
License: GPL (>= 2)
Encoding: UTF-8
RoxygenNote: 7.0.2
Depends: R (>= 3.5),
Imports: Rcpp (>= 0.12.12), BB, foreach, iterators, doSNOW, Matrix,
        parallel, MASS, locfit, fitdistrplus, stats, methods, graphics,
        grDevices, SingleCellExperiment, SummarizedExperiment,
        BiocParallel, utils
LinkingTo: Rcpp, RcppArmadillo,RcppProgress
Suggests: knitr, rmarkdown, BiocStyle, devtools, testthat
VignetteBuilder: knitr
biocViews: ImmunoOncology, Normalization, RNASeq, SingleCell,Sequencing
URL: https://github.com/WT215/bayNorm
BugReports: https://github.com/WT215/bayNorm/issues
git_url: https://git.bioconductor.org/packages/bayNorm
git_branch: RELEASE_3_22
git_last_commit: a368043
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 02:41:08 UTC; biocbuild
Author: Wenhao Tang [aut, cre],
  Fran<U+00E7>ois Bertaux [aut],
  Philipp Thomas [aut],
  Claire Stefanelli [aut],
  Malika Saint [aut],
  Samuel Marguerat [aut],
  Vahid Shahrezaei [aut]
Maintainer: Wenhao Tang <wt215@ic.ac.uk>
