# Generated by roxygen2: do not edit by hand

export(AdjustSIZE_fun)
export(BB_Fun)
export(BetaFun)
export(Check_input)
export(DownSampling)
export(EstPrior)
export(EstPrior_rcpp)
export(EstPrior_sprcpp)
export(NOISY_FUN)
export(Prior_fun)
export(SyntheticControl)
export(asMatrix)
export(as_matrix)
export(bayNorm)
export(bayNorm_sup)
export(noisy_gene_detection)
export(t_sp)
import(BiocParallel)
import(MASS)
import(doSNOW)
import(fitdistrplus)
import(foreach)
import(grDevices)
import(graphics)
import(iterators)
import(locfit)
import(methods)
import(parallel)
import(stats)
import(utils)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,t)
importFrom(Rcpp,sourceCpp)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
useDynLib(bayNorm)
