#' @title A subset of Grun et al (2014) data: 2i samples
#' @docType data
#' @description Small extract (20 genes and 74 cells)
#' from the Grun et al (2014) data: 2i samples
#' @format A list: EXAMPLE_DATA_list[[1]]: inputdata,
#' EXAMPLE_DATA_list[[2]]: inputbeta
#' (a vector of probabilities with length equal
#' to the number of cells), EXAMPLE_DATA_list[[3]]:
#' mu (MME method estimated mean expression for each gene),
#' EXAMPLE_DATA_list[[4]]: size (adjusted MME size
#' for each gene).
#'
#' @references
#'
#' Grun, Kester and van Oudenaarden (2014). Nature Methods.
#' @examples
#' data(EXAMPLE_DATA_list)
"EXAMPLE_DATA_list"
